aboutsummaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
-rw-r--r--Makefile3
-rw-r--r--cache.c45
-rw-r--r--cgit.c31
-rw-r--r--cgit.css4
-rw-r--r--cgit.h7
-rw-r--r--cgit.js68
-rw-r--r--cgitrc.5.txt35
-rw-r--r--cmd.c18
m---------git0
-rw-r--r--html.c2
-rw-r--r--robots.txt1
-rw-r--r--shared.c12
-rw-r--r--ui-atom.c34
-rw-r--r--ui-blame.c13
-rw-r--r--ui-commit.c3
-rw-r--r--ui-log.c8
-rw-r--r--ui-repolist.c2
-rw-r--r--ui-shared.c68
-rw-r--r--ui-shared.h2
-rw-r--r--ui-stats.c3
-rw-r--r--ui-tree.c28
21 files changed, 299 insertions, 88 deletions
diff --git a/Makefile b/Makefile
index d13c5bd..ad825be 100644
--- a/Makefile
+++ b/Makefile
@@ -14,7 +14,7 @@ htmldir = $(docdir)
pdfdir = $(docdir)
mandir = $(prefix)/share/man
SHA1_HEADER = <openssl/sha.h>
-GIT_VER = 2.32.0
+GIT_VER = 2.39.0
GIT_URL = https://www.kernel.org/pub/software/scm/git/git-$(GIT_VER).tar.xz
INSTALL = install
COPYTREE = cp -r
@@ -87,6 +87,7 @@ install: all
$(INSTALL) -m 0755 cgit $(DESTDIR)$(CGIT_SCRIPT_PATH)/$(CGIT_SCRIPT_NAME)
$(INSTALL) -m 0755 -d $(DESTDIR)$(CGIT_DATA_PATH)
$(INSTALL) -m 0644 cgit.css $(DESTDIR)$(CGIT_DATA_PATH)/cgit.css
+ $(INSTALL) -m 0644 cgit.js $(DESTDIR)$(CGIT_DATA_PATH)/cgit.js
$(INSTALL) -m 0644 cgit.png $(DESTDIR)$(CGIT_DATA_PATH)/cgit.png
$(INSTALL) -m 0644 favicon.ico $(DESTDIR)$(CGIT_DATA_PATH)/favicon.ico
$(INSTALL) -m 0644 robots.txt $(DESTDIR)$(CGIT_DATA_PATH)/robots.txt
diff --git a/cache.c b/cache.c
index 55199e8..1c843ba 100644
--- a/cache.c
+++ b/cache.c
@@ -85,40 +85,45 @@ static int close_slot(struct cache_slot *slot)
/* Print the content of the active cache slot (but skip the key). */
static int print_slot(struct cache_slot *slot)
{
+ off_t off;
#ifdef HAVE_LINUX_SENDFILE
- off_t start_off;
- int ret;
+ off_t size;
+#endif
+
+ off = slot->keylen + 1;
- start_off = slot->keylen + 1;
+#ifdef HAVE_LINUX_SENDFILE
+ size = slot->cache_st.st_size;
do {
- ret = sendfile(STDOUT_FILENO, slot->cache_fd, &start_off,
- slot->cache_st.st_size - start_off);
+ ssize_t ret;
+ ret = sendfile(STDOUT_FILENO, slot->cache_fd, &off, size - off);
if (ret < 0) {
if (errno == EAGAIN || errno == EINTR)
continue;
+ /* Fall back to read/write on EINVAL or ENOSYS */
+ if (errno == EINVAL || errno == ENOSYS)
+ break;
return errno;
}
- return 0;
+ if (off == size)
+ return 0;
} while (1);
-#else
- ssize_t i, j;
+#endif
- i = lseek(slot->cache_fd, slot->keylen + 1, SEEK_SET);
- if (i != slot->keylen + 1)
+ if (lseek(slot->cache_fd, off, SEEK_SET) != off)
return errno;
do {
- i = j = xread(slot->cache_fd, slot->buf, sizeof(slot->buf));
- if (i > 0)
- j = xwrite(STDOUT_FILENO, slot->buf, i);
- } while (i > 0 && j == i);
-
- if (i < 0 || j != i)
- return errno;
- else
- return 0;
-#endif
+ ssize_t ret;
+ ret = xread(slot->cache_fd, slot->buf, sizeof(slot->buf));
+ if (ret < 0)
+ return errno;
+ if (ret == 0)
+ return 0;
+ if (write_in_full(STDOUT_FILENO, slot->buf, ret) < 0)
+ return errno;
+ } while (1);
}
/* Check if the slot has expired */
diff --git a/cgit.css b/cgit.css
index dfa144d..1b848cf 100644
--- a/cgit.css
+++ b/cgit.css
@@ -363,6 +363,10 @@ div#cgit table.blame td.lines > div > pre {
top: 0;
}
+div#cgit table.blame .oid {
+ font-size: 100%;
+}
+
div#cgit table.bin-blob {
margin-top: 0.5em;
border: solid 1px black;
diff --git a/cgit.h b/cgit.h
index 69b5c13..9f9daac 100644
--- a/cgit.h
+++ b/cgit.h
@@ -25,6 +25,7 @@
#include <utf8.h>
#include <notes.h>
#include <graph.h>
+#include <inttypes.h>
/* Add isgraph(x) to Git's sane ctype support (see git-compat-util.h) */
#undef isgraph
@@ -86,6 +87,7 @@ struct cgit_repo {
char *owner;
char *homepage;
char *defbranch;
+ char *default_page;
char *module_link;
struct string_list readme;
char *section;
@@ -195,7 +197,7 @@ struct cgit_config {
char *cache_root;
char *clone_prefix;
char *clone_url;
- char *css;
+ char *default_page;
char *favicon;
char *footer;
char *head_include;
@@ -206,10 +208,12 @@ struct cgit_config {
char *module_link;
char *project_list;
struct string_list readme;
+ struct string_list css;
char *robots;
char *root_title;
char *root_desc;
char *root_readme;
+ char *root_default_page;
char *script_name;
char *section;
char *repository_sort;
@@ -264,6 +268,7 @@ struct cgit_config {
int branch_sort;
int commit_sort;
struct string_list mimetypes;
+ struct string_list js;
struct cgit_filter *about_filter;
struct cgit_filter *commit_filter;
struct cgit_filter *source_filter;
diff --git a/cgit.js b/cgit.js
new file mode 100644
index 0000000..df3ad4e
--- /dev/null
+++ b/cgit.js
@@ -0,0 +1,68 @@
+/* cgit.js: javacript functions for cgit
+ *
+ * Copyright (C) 2006-2018 cgit Development Team <cgit@lists.zx2c4.com>
+ *
+ * Licensed under GNU General Public License v2
+ * (see COPYING for full license text)
+ */
+
+(function () {
+
+/* This follows the logic and suffixes used in ui-shared.c */
+
+var age_classes = [ "age-mins", "age-hours", "age-days", "age-weeks", "age-months", "age-years" ];
+var age_suffix = [ "min.", "hours", "days", "weeks", "months", "years", "years" ];
+var age_next = [ 60, 3600, 24 * 3600, 7 * 24 * 3600, 30 * 24 * 3600, 365 * 24 * 3600, 365 * 24 * 3600 ];
+var age_limit = [ 7200, 24 * 7200, 7 * 24 * 7200, 30 * 24 * 7200, 365 * 25 * 7200, 365 * 25 * 7200 ];
+var update_next = [ 10, 5 * 60, 1800, 24 * 3600, 24 * 3600, 24 * 3600, 24 * 3600 ];
+
+function render_age(e, age) {
+ var t, n;
+
+ for (n = 0; n < age_classes.length; n++)
+ if (age < age_limit[n])
+ break;
+
+ t = Math.round(age / age_next[n]) + " " + age_suffix[n];
+
+ if (e.textContent != t) {
+ e.textContent = t;
+ if (n == age_classes.length)
+ n--;
+ if (e.className != age_classes[n])
+ e.className = age_classes[n];
+ }
+}
+
+function aging() {
+ var n, next = 24 * 3600,
+ now_ut = Math.round((new Date().getTime() / 1000));
+
+ for (n = 0; n < age_classes.length; n++) {
+ var m, elems = document.getElementsByClassName(age_classes[n]);
+
+ if (elems.length && update_next[n] < next)
+ next = update_next[n];
+
+ for (m = 0; m < elems.length; m++) {
+ var age = now_ut - elems[m].getAttribute("data-ut");
+
+ render_age(elems[m], age);
+ }
+ }
+
+ /*
+ * We only need to come back when the age might have changed.
+ * Eg, if everything is counted in hours already, once per
+ * 5 minutes is accurate enough.
+ */
+
+ window.setTimeout(aging, next * 1000);
+}
+
+document.addEventListener("DOMContentLoaded", function() {
+ /* we can do the aging on DOM content load since no layout dependency */
+ aging();
+}, false);
+
+})();
diff --git a/cgitrc.5.txt b/cgitrc.5.txt
index 33a6a8c..ac86864 100644
--- a/cgitrc.5.txt
+++ b/cgitrc.5.txt
@@ -126,7 +126,14 @@ commit-sort::
css::
Url which specifies the css document to include in all cgit pages.
- Default value: "/cgit.css".
+ Default value: "/cgit.css". May be given multiple times, each
+ css URL path is added in the head section of the document in turn.
+
+default-page::
+ Specifies the default page for repositories. This setting is only used
+ if `repo.default-page` is unspecified. Possible values: "about",
+ "summary", "refs", "log", "tree", "commit", "diff", "stats". Default
+ value: "summary".
email-filter::
Specifies a command which will be invoked to format names and email
@@ -238,6 +245,11 @@ include::
Name of a configfile to include before the rest of the current config-
file is parsed. Default value: none. See also: "MACRO EXPANSION".
+js::
+ Url which specifies the javascript script document to include in all cgit
+ pages. Default value: "/cgit.js". Setting this to an empty string will
+ disable generation of the link to this file in the head section.
+
local-time::
Flag which, if set to "1", makes cgit print commit and tag times in the
servers timezone. Default value: "0".
@@ -269,7 +281,8 @@ max-message-length::
max-repo-count::
Specifies the number of entries to list per page on the repository
- index page. Default value: "50".
+ index page. The value "0" shows all repositories without limitation.
+ Default value: "50".
max-repodesc-length::
Specifies the maximum number of repo description characters to display
@@ -352,6 +365,10 @@ robots::
Text used as content for the "robots" meta-tag. Default value:
"index, nofollow".
+root-default-page::
+ Specifies the default root page. Possible values are "repolist" and
+ "about". Default value: "repolist".
+
root-desc::
Text printed below the heading on the repository index page. Default
value: "a fast webinterface for the git dscm".
@@ -475,6 +492,10 @@ repo.commit-sort::
ordering. If unset, the default ordering of "git log" is used. Default
value: unset.
+repo.default-page::
+ Specifies the default page for the repository. Default value: global
+ default-page.
+
repo.defbranch::
The name of the default branch for this repository. If no such branch
exists in the repository, the first branch name (when sorted) is used
@@ -579,11 +600,11 @@ repo.readme::
verbatim as the "About" page for this repo. You may also specify a
git refspec by head or by hash by prepending the refspec followed by
a colon. For example, "master:docs/readme.mkd". If the value begins
- with a colon, i.e. ":docs/readme.rst", the default branch of the
- repository will be used. Sharing any file will expose that entire
- directory tree to the "/about/PATH" endpoints, so be sure that there
- are no non-public files located in the same directory as the readme
- file. Default value: <readme>.
+ with a colon, i.e. ":docs/readme.rst", the head giving in query or
+ the default branch of the repository will be used. Sharing any file
+ will expose that entire directory tree to the "/about/PATH" endpoints,
+ so be sure that there are no non-public files located in the same
+ directory as the readme file. Default value: <readme>.
repo.section::
Override the current section name for this repository. Default value:
diff --git a/cmd.c b/cmd.c
index 0eb75b1..669c345 100644
--- a/cmd.c
+++ b/cmd.c
@@ -51,13 +51,10 @@ static void about_fn(void)
free(redirect);
} else if (ctx.repo->readme.nr)
cgit_print_repo_readme(ctx.qry.path);
- else if (ctx.repo->homepage)
- cgit_redirect(ctx.repo->homepage, false);
else {
- char *currenturl = cgit_currenturl();
- char *redirect = fmtalloc("%s../", currenturl);
+ char *redirect = fmtalloc("%s%s/summary/",
+ ctx.cfg.virtual_root, ctx.repo->url);
cgit_redirect(redirect, false);
- free(currenturl);
free(redirect);
}
} else
@@ -195,10 +192,13 @@ struct cgit_cmd *cgit_get_cmd(void)
int i;
if (ctx.qry.page == NULL) {
- if (ctx.repo)
- ctx.qry.page = "summary";
- else
- ctx.qry.page = "repolist";
+ if (ctx.repo) {
+ if (ctx.repo->default_page && *ctx.repo->default_page)
+ ctx.qry.page = ctx.repo->default_page;
+ else
+ ctx.qry.page = ctx.cfg.default_page;
+ } else
+ ctx.qry.page = ctx.cfg.root_default_page;
}
for (i = 0; i < sizeof(cmds)/sizeof(*cmds); i++)
diff --git a/git b/git
-Subproject ebf3c04b262aa27fbb97f8a0156c2347fecafaf
+Subproject c48035d29b4e524aed3a32f0403676f0d912886
diff --git a/html.c b/html.c
index 7f81965..0bac34b 100644
--- a/html.c
+++ b/html.c
@@ -59,7 +59,7 @@ char *fmt(const char *format, ...)
va_start(args, format);
len = vsnprintf(buf[bufidx], sizeof(buf[bufidx]), format, args);
va_end(args);
- if (len > sizeof(buf[bufidx])) {
+ if (len >= sizeof(buf[bufidx])) {
fprintf(stderr, "[html.c] string truncated: %s\n", format);
exit(1);
}
diff --git a/robots.txt b/robots.txt
index 4ce948f..1b33266 100644
--- a/robots.txt
+++ b/robots.txt
@@ -1,3 +1,4 @@
User-agent: *
Disallow: /*/snapshot/*
+Disallow: /*/blame/*
Allow: /
diff --git a/shared.c b/shared.c
index 8115469..0bceb98 100644
--- a/shared.c
+++ b/shared.c
@@ -341,9 +341,8 @@ void cgit_diff_tree(const struct object_id *old_oid,
filepair_fn fn, const char *prefix, int ignorews)
{
struct diff_options opt;
- struct pathspec_item item;
+ struct pathspec_item *item;
- memset(&item, 0, sizeof(item));
diff_setup(&opt);
opt.output_format = DIFF_FORMAT_CALLBACK;
opt.detect_rename = 1;
@@ -354,10 +353,11 @@ void cgit_diff_tree(const struct object_id *old_oid,
opt.format_callback = cgit_diff_tree_cb;
opt.format_callback_data = fn;
if (prefix) {
- item.match = xstrdup(prefix);
- item.len = strlen(prefix);
+ item = xcalloc(1, sizeof(*item));
+ item->match = xstrdup(prefix);
+ item->len = strlen(prefix);
opt.pathspec.nr = 1;
- opt.pathspec.items = &item;
+ opt.pathspec.items = item;
}
diff_setup_done(&opt);
@@ -367,8 +367,6 @@ void cgit_diff_tree(const struct object_id *old_oid,
diff_root_tree_oid(new_oid, "", &opt);
diffcore_std(&opt);
diff_flush(&opt);
-
- free(item.match);
}
void cgit_diff_commit(struct commit *commit, filepair_fn fn, const char *prefix)
diff --git a/ui-atom.c b/ui-atom.c
index 1056f36..5f4ad7d 100644
--- a/ui-atom.c
+++ b/ui-atom.c
@@ -67,17 +67,12 @@ static void add_entry(struct commit *commit, const char *host)
html("'/>\n");
free(pageurl);
}
- htmlf("<id>%s</id>\n", hex);
+ html("<id>");
+ html_txtf("urn:%s:%s", the_hash_algo->name, hex);
+ html("</id>\n");
html("<content type='text'>\n");
html_txt(info->msg);
html("</content>\n");
- html("<content type='xhtml'>\n");
- html("<div xmlns='http://www.w3.org/1999/xhtml'>\n");
- html("<pre>\n");
- html_txt(info->msg);
- html("</pre>\n");
- html("</div>\n");
- html("</content>\n");
html("</entry>\n");
cgit_free_commitinfo(info);
}
@@ -90,6 +85,7 @@ void cgit_print_atom(char *tip, const char *path, int max_count)
struct commit *commit;
struct rev_info rev;
int argc = 2;
+ bool first = true;
if (ctx.qry.show_all)
argv[1] = "--all";
@@ -130,18 +126,30 @@ void cgit_print_atom(char *tip, const char *path, int max_count)
html_txt(ctx.repo->desc);
html("</subtitle>\n");
if (host) {
+ char *fullurl = cgit_currentfullurl();
char *repourl = cgit_repourl(ctx.repo->url);
+ html("<id>");
+ html_txtf("%s%s%s", cgit_httpscheme(), host, fullurl);
+ html("</id>\n");
+ html("<link rel='self' href='");
+ html_attrf("%s%s%s", cgit_httpscheme(), host, fullurl);
+ html("'/>\n");
html("<link rel='alternate' type='text/html' href='");
- html(cgit_httpscheme());
- html_attr(host);
- html_attr(repourl);
+ html_attrf("%s%s%s", cgit_httpscheme(), host, repourl);
html("'/>\n");
+ free(fullurl);
free(repourl);
}
while ((commit = get_revision(&rev)) != NULL) {
+ if (first) {
+ html("<updated>");
+ html_txt(show_date(commit->date, 0,
+ date_mode_from_type(DATE_ISO8601_STRICT)));
+ html("</updated>\n");
+ first = false;
+ }
add_entry(commit, host);
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
html("</feed>\n");
diff --git a/ui-blame.c b/ui-blame.c
index 03136f7..aedce8d 100644
--- a/ui-blame.c
+++ b/ui-blame.c
@@ -54,6 +54,15 @@ static void emit_blame_entry_hash(struct blame_entry *ent)
html("</span>");
free(detail);
+ if (!parse_commit(suspect->commit) && suspect->commit->parents) {
+ struct commit *parent = suspect->commit->parents->item;
+
+ html(" ");
+ cgit_blame_link("^", "Blame the previous revision", NULL,
+ ctx.qry.head, oid_to_hex(&parent->object.oid),
+ suspect->path);
+ }
+
while (line++ < ent->num_lines)
html("\n");
}
@@ -152,6 +161,10 @@ static void print_object(const struct object_id *oid, const char *path,
cgit_tree_link("tree", NULL, NULL, ctx.qry.head, rev, path);
html(")\n");
+ if (buffer_is_binary(buf, size)) {
+ html("<div class='error'>blob is binary.</div>");
+ goto cleanup;
+ }
if (ctx.cfg.max_blob_size && size / 1024 > ctx.cfg.max_blob_size) {
htmlf("<div class='error'>blob size (%ldKB)"
" exceeds display size limit (%dKB).</div>",
diff --git a/ui-commit.c b/ui-commit.c
index 948118c..0787237 100644
--- a/ui-commit.c
+++ b/ui-commit.c
@@ -39,10 +39,11 @@ void cgit_print_commit(char *hex, const char *prefix)
}
info = cgit_parse_commit(commit);
- format_display_notes(&oid, &notes, PAGE_ENCODING, 0);
+ format_display_notes(&oid, &notes, PAGE_ENCODING, 1);
load_ref_decorations(NULL, DECORATE_FULL_REFS);
+ ctx.page.title = fmtalloc("%s - %s", info->subject, ctx.page.title);
cgit_print_layout_start();
cgit_print_diff_ctrls();
html("<table summary='commit info' class='commit-info'>\n");
diff --git a/ui-log.c b/ui-log.c
index 20774bf..311304a 100644
--- a/ui-log.c
+++ b/ui-log.c
@@ -159,7 +159,7 @@ static int show_commit(struct commit *commit, struct rev_info *revs)
"", &revs->diffopt);
diffcore_std(&revs->diffopt);
- found = !diff_queue_is_empty();
+ found = !diff_queue_is_empty(&revs->diffopt);
saved_fmt = revs->diffopt.output_format;
revs->diffopt.output_format = DIFF_FORMAT_CALLBACK;
revs->diffopt.format_callback = cgit_diff_tree_cb;
@@ -489,8 +489,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
for (i = 0; i < ofs && (commit = get_revision(&rev)) != NULL; /* nop */) {
if (show_commit(commit, &rev))
i++;
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
@@ -511,8 +510,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
i++;
print_commit(commit, &rev);
}
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
if (pager) {
diff --git a/ui-repolist.c b/ui-repolist.c
index 529a203..2390eae 100644
--- a/ui-repolist.c
+++ b/ui-repolist.c
@@ -321,7 +321,7 @@ void cgit_print_repolist(void)
}
htmlf("<tr><td class='%s'>",
!sorted && section ? "sublevel-repo" : "toplevel-repo");
- cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
+ cgit_repo_link(ctx.repo->name, NULL, NULL, NULL);
html("</td><td>");
repourl = cgit_repourl(ctx.repo->url);
html_link_open(repourl, NULL, NULL);
diff --git a/ui-shared.c b/ui-shared.c
index acd8ab5..81e1f46 100644
--- a/ui-shared.c
+++ b/ui-shared.c
@@ -328,10 +328,16 @@ static void reporevlink(const char *page, const char *name, const char *title,
html("</a>");
}
+void cgit_repo_link(const char *name, const char *title, const char *class,
+ const char *head)
+{
+ reporevlink(NULL, name, title, class, head, NULL, NULL);
+}
+
void cgit_summary_link(const char *name, const char *title, const char *class,
const char *head)
{
- reporevlink(NULL, name, title, class, head, NULL, NULL);
+ reporevlink("summary", name, title, class, head, NULL, NULL);
}
void cgit_tag_link(const char *name, const char *title, const char *class,
@@ -673,7 +679,7 @@ const struct date_mode *cgit_date_mode(enum date_mode_type type)
static void print_rel_date(time_t t, int tz, double value,
const char *class, const char *suffix)
{
- htmlf("<span class='%s' title='", class);
+ htmlf("<span class='%s' data-ut='%" PRIu64 "' title='", class, (uint64_t)t);
html_attr(show_date(t, tz, cgit_date_mode(DATE_ISO8601)));
htmlf("'>%.0f %s</span>", value, suffix);
}
@@ -768,6 +774,38 @@ static void print_rel_vcs_link(const char *url)
html(" Git repository'/>\n");
}
+static int emit_css_link(struct string_list_item *s, void *arg)
+{
+ /* Do not emit anything if css= is specified. */
+ if (s && *s->string == '\0')
+ return 0;
+
+ html("<link rel='stylesheet' type='text/css' href='");
+ if (s)
+ html_attr(s->string);
+ else
+ html_attr((const char *)arg);
+ html("'/>\n");
+
+ return 0;
+}
+
+static int emit_js_link(struct string_list_item *s, void *arg)
+{
+ /* Do not emit anything if js= is specified. */
+ if (s && *s->string == '\0')
+ return 0;
+
+ html("<script type='text/javascript' src='");
+ if (s)
+ html_attr(s->string);
+ else
+ html_attr((const char *)arg);
+ html("'></script>\n");
+
+ return 0;
+}
+
void cgit_print_docstart(void)
{
char *host = cgit_hosturl();
@@ -787,9 +825,17 @@ void cgit_print_docstart(void)
htmlf("<meta name='generator' content='cgit %s'/>\n", cgit_version);
if (ctx.cfg.robots && *ctx.cfg.robots)
htmlf("<meta name='robots' content='%s'/>\n", ctx.cfg.robots);
- html("<link rel='stylesheet' type='text/css' href='");
- html_attr(ctx.cfg.css);
- html("'/>\n");
+
+ if (ctx.cfg.css.items)
+ for_each_string_list(&ctx.cfg.css, emit_css_link, NULL);
+ else
+ emit_css_link(NULL, "/cgit.css");
+
+ if (ctx.cfg.js.items)
+ for_each_string_list(&ctx.cfg.js, emit_js_link, NULL);
+ else
+ emit_js_link(NULL, "/cgit.js");
+
if (ctx.cfg.favicon) {
html("<link rel='shortcut icon' href='");
html_attr(ctx.cfg.favicon);
@@ -995,7 +1041,7 @@ static void print_header(void)
if (ctx.repo) {
cgit_index_link("index", NULL, NULL, NULL, NULL, 0, 1);
html(" : ");
- cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
+ cgit_repo_link(ctx.repo->name, NULL, NULL, NULL);
if (ctx.env.authenticated) {
html("</td><td class='form'>");
html("<form method='get'>\n");
@@ -1016,7 +1062,13 @@ static void print_header(void)
if (ctx.repo) {
html_txt(ctx.repo->desc);
html("</td><td class='sub right'>");
- html_txt(ctx.repo->owner);
+ if (ctx.repo->owner_filter) {
+ cgit_open_filter(ctx.repo->owner_filter);
+ html_txt(ctx.repo->owner);
+ cgit_close_filter(ctx.repo->owner_filter);
+ } else {
+ html_txt(ctx.repo->owner);
+ }
} else {
if (ctx.cfg.root_desc)
html_txt(ctx.cfg.root_desc);
@@ -1084,7 +1136,7 @@ void cgit_print_pageheader(void)
html("</form>\n");
} else if (ctx.env.authenticated) {
char *currenturl = cgit_currenturl();
- site_link(NULL, "index", NULL, hc("repolist"), NULL, NULL, 0, 1);
+ site_link("repolist", "index", NULL, hc("repolist"), NULL, NULL, 0, 1);
if (ctx.cfg.root_readme)
site_link("about", "about", NULL, hc("about"),
NULL, NULL, 0, 1);
diff --git a/ui-shared.h b/ui-shared.h
index 6964873..4d14858 100644
--- a/ui-shared.h
+++ b/ui-shared.h
@@ -17,6 +17,8 @@ extern void cgit_add_clone_urls(void (*fn)(const char *));
extern void cgit_index_link(const char *name, const char *title,
const char *class, const char *pattern, const char *sort, int ofs, int always_root);
+extern void cgit_repo_link(const char *name, const char *title,
+ const char *class, const char *head);
extern void cgit_summary_link(const char *name, const char *title,
const char *class, const char *head);
extern void cgit_tag_link(const char *name, const char *title,
diff --git a/ui-stats.c b/ui-stats.c
index 09b3625..40ed6c2 100644
--- a/ui-stats.c
+++ b/ui-stats.c
@@ -241,8 +241,7 @@ static struct string_list collect_stats(const struct cgit_period *period)
memset(&authors, 0, sizeof(authors));
while ((commit = get_revision(&rev)) != NULL) {
add_commit(&authors, commit, period);
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
return authors;
diff --git a/ui-tree.c b/ui-tree.c
index b61f6f5..98ce1ca 100644
--- a/ui-tree.c
+++ b/ui-tree.c
@@ -89,6 +89,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
enum object_type type;
char *buf;
unsigned long size;
+ bool is_binary;
type = oid_object_info(the_repository, oid, &size);
if (type == OBJ_BAD) {
@@ -103,6 +104,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
"Error reading object %s", oid_to_hex(oid));
return;
}
+ is_binary = buffer_is_binary(buf, size);
cgit_set_title_from_path(path);
@@ -110,7 +112,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
htmlf("blob: %s (", oid_to_hex(oid));
cgit_plain_link("plain", NULL, NULL, ctx.qry.head,
rev, path);
- if (ctx.repo->enable_blame) {
+ if (ctx.repo->enable_blame && !is_binary) {
html(") (");
cgit_blame_link("blame", NULL, NULL, ctx.qry.head,
rev, path);
@@ -123,7 +125,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
return;
}
- if (buffer_is_binary(buf, size))
+ if (is_binary)
print_binary_buffer(buf, size);
else
print_text_buffer(basename, buf, size);
@@ -202,9 +204,11 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
struct walk_tree_context *walk_tree_ctx = cbdata;
char *name;
struct strbuf fullpath = STRBUF_INIT;
+ struct strbuf linkpath = STRBUF_INIT;
struct strbuf class = STRBUF_INIT;
enum object_type type;
unsigned long size = 0;
+ char *buf;
name = xstrdup(pathname);
strbuf_addf(&fullpath, "%s%s%s", ctx.qry.path ? ctx.qry.path : "",
@@ -216,8 +220,7 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
htmlf("<tr><td colspan='3'>Bad object: %s %s</td></tr>",
name,
oid_to_hex(oid));
- free(name);
- return 0;
+ goto cleanup;
}
}
@@ -237,6 +240,21 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
cgit_tree_link(name, NULL, class.buf, ctx.qry.head,
walk_tree_ctx->curr_rev, fullpath.buf);
}
+ if (S_ISLNK(mode)) {
+ html(" -> ");
+ buf = read_object_file(oid, &type, &size);
+ if (!buf) {
+ htmlf("Error reading object: %s", oid_to_hex(oid));
+ goto cleanup;
+ }
+ strbuf_addbuf(&linkpath, &fullpath);
+ strbuf_addf(&linkpath, "/../%s", buf);
+ strbuf_normalize_path(&linkpath);
+ cgit_tree_link(buf, NULL, class.buf, ctx.qry.head,
+ walk_tree_ctx->curr_rev, linkpath.buf);
+ free(buf);
+ strbuf_release(&linkpath);
+ }
htmlf("</td><td class='ls-size'>%li</td>", size);
html("<td>");
@@ -253,6 +271,8 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
cgit_blame_link("blame", NULL, "button", ctx.qry.head,
walk_tree_ctx->curr_rev, fullpath.buf);
html("</td></tr>\n");
+
+cleanup:
free(name);
strbuf_release(&fullpath);
strbuf_release(&class);
diff --git a/cgit.c b/cgit.c
index 08d81a1..e7c0172 100644
--- a/cgit.c
+++ b/cgit.c
@@ -56,6 +56,8 @@ static void repo_config(struct cgit_repo *repo, const char *name, const char *va
repo->homepage = xstrdup(value);
else if (!strcmp(name, "defbranch"))
repo->defbranch = xstrdup(value);
+ else if (!strcmp(name, "default-page"))
+ repo->default_page = xstrdup(value);
else if (!strcmp(name, "extra-head-content"))
repo->extra_head_content = xstrdup(value);
else if (!strcmp(name, "snapshots"))
@@ -141,8 +143,12 @@ static void config_cb(const char *name, const char *value)
ctx.cfg.root_desc = xstrdup(value);
else if (!strcmp(name, "root-readme"))
ctx.cfg.root_readme = xstrdup(value);
+ else if (!strcmp(name, "root-default-page"))
+ ctx.cfg.root_default_page = xstrdup(value);
else if (!strcmp(name, "css"))
- ctx.cfg.css = xstrdup(value);
+ string_list_append(&ctx.cfg.css, xstrdup(value));
+ else if (!strcmp(name, "js"))
+ string_list_append(&ctx.cfg.js, xstrdup(value));
else if (!strcmp(name, "favicon"))
ctx.cfg.favicon = xstrdup(value);
else if (!strcmp(name, "footer"))
@@ -155,6 +161,8 @@ static void config_cb(const char *name, const char *value)
ctx.cfg.logo = xstrdup(value);
else if (!strcmp(name, "logo-link"))
ctx.cfg.logo_link = xstrdup(value);
+ else if (!strcmp(name, "default-page"))
+ ctx.cfg.default_page = xstrdup(value);
else if (!strcmp(name, "module-link"))
ctx.cfg.module_link = xstrdup(value);
else if (!strcmp(name, "strict-export"))
@@ -237,9 +245,11 @@ static void config_cb(const char *name, const char *value)
ctx.cfg.max_repodesc_len = atoi(value);
else if (!strcmp(name, "max-blob-size"))
ctx.cfg.max_blob_size = atoi(value);
- else if (!strcmp(name, "max-repo-count"))
+ else if (!strcmp(name, "max-repo-count")) {
ctx.cfg.max_repo_count = atoi(value);
- else if (!strcmp(name, "max-commit-count"))
+ if (ctx.cfg.max_repo_count <= 0)
+ ctx.cfg.max_repo_count = INT_MAX;
+ } else if (!strcmp(name, "max-commit-count"))
ctx.cfg.max_commit_count = atoi(value);
else if (!strcmp(name, "project-list"))
ctx.cfg.project_list = xstrdup(expand_macros(value));
@@ -376,7 +386,7 @@ static void prepare_context(void)
ctx.cfg.case_sensitive_sort = 1;
ctx.cfg.branch_sort = 0;
ctx.cfg.commit_sort = 0;
- ctx.cfg.css = "/cgit.css";
+ ctx.cfg.default_page= "summary";
ctx.cfg.logo = "/cgit.png";
ctx.cfg.favicon = "/favicon.ico";
ctx.cfg.local_time = 0;
@@ -397,6 +407,7 @@ static void prepare_context(void)
ctx.cfg.robots = "index, nofollow";
ctx.cfg.root_title = "Git repository browser";
ctx.cfg.root_desc = "a fast webinterface for the git dscm";
+ ctx.cfg.root_default_page = "repolist";
ctx.cfg.scan_hidden_path = 0;
ctx.cfg.script_name = CGIT_SCRIPT_NAME;
ctx.cfg.section = "";
@@ -428,7 +439,7 @@ static void prepare_context(void)
ctx.page.modified = time(NULL);
ctx.page.expires = ctx.page.modified;
ctx.page.etag = NULL;
- string_list_init(&ctx.cfg.mimetypes, 1);
+ string_list_init_dup(&ctx.cfg.mimetypes);
if (ctx.env.script_name)
ctx.cfg.script_name = xstrdup(ctx.env.script_name);
if (ctx.env.query_string)
@@ -507,9 +518,11 @@ static inline void parse_readme(const char *readme, char **filename, char **ref,
/* Check if the readme is tracked in the git repo. */
colon = strchr(readme, ':');
if (colon && strlen(colon) > 1) {
- /* If it starts with a colon, we want to use
- * the default branch */
- if (colon == readme && repo->defbranch)
+ /* If it starts with a colon, we want to use head given
+ * from query or the default branch */
+ if (colon == readme && ctx.qry.head)
+ *ref = xstrdup(ctx.qry.head);
+ else if (colon == readme && repo->defbranch)
*ref = xstrdup(repo->defbranch);
else
*ref = xstrndup(readme, colon - readme);
@@ -806,6 +819,8 @@ static void print_repo(FILE *f, struct cgit_repo *repo)
}
if (repo->defbranch)
fprintf(f, "repo.defbranch=%s\n", repo->defbranch);
+ if (repo->default_page)
+ fprintf(f, "repo.default-page=%s\n", repo->default_page);
if (repo->extra_head_content)
fprintf(f, "repo.extra-head-content=%s\n", repo->extra_head_content);
if (repo->module_link)